package com.lol.comm;

import java.io.*;
import java.net.Socket;
import java.util.*;

import com.lol.db.*;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: </p>
 *
 * <p>
 * Classe per l'implementazione del singolo thread di comunicazione
 * Gestisce integralmente l'interazione col server attraverso un
 * meccanismo a stati
 * </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class Listener  extends Thread   {

    public int DoTest(int val)
    {
        return 1;
    }


    /**
     * usersHash
     * Riferimento agli utenti presenti sul server. Gli accessi a
     * questo sono tutti syynchronized
     */
    protected HashMap usersHash;

    /**
     * destinationsArray
     * Riferimento alle destinazioni globali presenti sul server. Gli accessi a
     * questo sono tutti syynchronized
     */
    protected ArrayList destinationsArray;

    /**
     * curDestination
     * La destinazione correntemente selezionata
     */
    protected Destinations curDestination;

    /**
     * curEvaluations
     * La valutazione correntemente selezionata dall'utente
     */
    protected Evaluations curEvaluations;

    /**
     * curUser
     * Utente connesso con il thread
     */
    protected Users curUser;

    /**
     * curState
     * Lo stato della macchina stati per la comunicazione client server
     * 0 Attende input
     * 1 Login/aggiunta utente
     * 2 Inserimento destinazione
     * 3 Lista delle destinazioni possibili
     * 7 Lista delle valutazioni per la meta selezionata
     * 5 Verifica inserimento nuova valutazione
     * 6 Effettivo inserimento della valutazione
     */
    protected int curState;

    /**
     * waitForVars
     * Indica se il programma server si aspetti o meno delle comunicazioni dal
     * client
     */
    protected int waitForVars;

    /**
     * varsToWaitFor
     * Contenitore per le variabili necessarie al funzionamento dello stato successivo
     * del listener
     */
    protected String varsToWaitFor[] = new String[4];

    /**
     * messages
     * Stringa di messaggi, per errori e comunicazioni eccezionali
     */
    protected String messages="";

    /**
     * is
     * Input stream, fonte di dati
     */
    DataInputStream is = null;

    /**
     * os
     * Output del server
     */
    PrintStream os = null;

    /**
     * clientSocket
     * La connessione principale
     */
    Socket clientSocket = null;

    /**
     * Costruttore di default del listener
     *
     * @param clis Socket  Il socket del listener corrente
     * @param users HashMap  hashmap degli utenti (per l'autenticazione)
     * @param destinations ArrayList array delle destinazioni
     * @param bestDestination Destinations indice della miglior destinazione presente
     */
    public Listener(Socket clis,HashMap users,ArrayList destinations,Destinations bestDestination,Listener connectedListeners[], int self){
        clientSocket=clis;
        usersHash = users;
        destinationsArray=destinations;
        selfListIndex=self;
        connectedListenersArray=connectedListeners;
    }

    int selfListIndex;
    Listener connectedListenersArray[];
    /**
     * Inserimento di un nuovo utente (se non presente) e comunque sua autenticazione
     *
     * @param name String  nome utente
     * @param pwd String  password
     * @return Users utente creato/trovato
     */
    protected synchronized Users insertUser(String name,String pwd)
    {
        Users curUser = null;
        if(usersHash.containsKey(name)){
            curUser=(Users)usersHash.get(name);
            if(curUser.getPwd().equalsIgnoreCase(pwd))
                return curUser;
            return null;
        }else{
            curUser=new Users();
            curUser.setLogin(name);
            curUser.setPwd(name);
            usersHash.put(name,curUser);
        }
        return curUser;
    }

    /**
     * Aggiunge una nuova destinazione e segnala un messaggio di errore
     * se gia presente
     *
     * @param name String il nome della destinazione da inserire
     * @return Destinations la destinazione inserita/trovata
     */
    protected synchronized Destinations insertDestination(String name )
    {
        Destinations curDest = null;
        for(int i=0;i<destinationsArray.size();i++){
            curDest=(Destinations)destinationsArray.get(i);
            if(curDest.getName().equalsIgnoreCase(name)){
                messages="Attenzione! Meta gia'presente!";
                return curDest;
            }
        }
        curDest=new Destinations();
        curDest.setName(name);
        destinationsArray.add(curDest);

        return curDest;
    }

    /**
     * Ottiene la lista delle destinazioni, crea delle stringhe in modo da avere
     * controllo sull'elemento synchronized il meno possibile
     *
     * @return ArrayList
     */
    protected synchronized ArrayList listDestinations( )
    {
        ArrayList arl = new ArrayList();
        Destinations curDest = null;
        for(int i=0;i<destinationsArray.size();i++){
            curDest = (Destinations)destinationsArray.get(i);
            arl.add(String.valueOf(i)+"-"+curDest.getName());
        }
        return arl;
    }

    /**
     * getDestination
     * Dato il numero della destinazione sull'arraylist delle destinazioni del
     * server ne restituisce l'istanza
     *
     * @param number int L'indice della destinazione
     * @return Destinations La destinazione prescelta (null se non presente)
     */
    protected synchronized Destinations getDestination(int number)
    {

        if(number <destinationsArray.size())
            return (Destinations)destinationsArray.get(number);
        messages = "Attenzione meta inesistente!";
        return null;
    }

    /**
         * getDestination
         * Data la descrizione della destinazione sull'arraylist delle destinazioni del
         * server ne restituisce l'istanza
         *
         * @param name String il nome della destinazione
         * @return Destinations La destinazione prescelta (null se non presente)
         */
        protected synchronized Destinations getDestination(String name)
        {
            for(int i=0;i<destinationsArray.size();i++){
                if(((Destinations)destinationsArray.get(i)).getName().equalsIgnoreCase(name))
                    return (Destinations)destinationsArray.get(i);
            }

            messages = "Attenzione meta inesistente!";
            return null;
    }

    /**
     * Aggiunge una valutazione ad destinazione, inibisce la possibilita'per un utente
     * di inserire mete doppie
     *
     * @param dest Destinations
     * @param voto String
     * @param giudizio String
     */
    protected synchronized void insertEvaluation(Destinations dest,String voto,String giudizio)
    {
        if(curUser.InsertEvaluation(dest,voto,giudizio)){
            messages="Attenzione Meta gia'valutat!!";
            return;
        }

        if(TheServer.GetBestDestinations()==null)
            TheServer.SetBestDestinations(dest);
        if(dest.getAverage() > TheServer.GetBestDestinations().getAverage())
            TheServer.SetBestDestinations(dest);
    }

    /**
     * Data una destinazione ne fornisce la lista, sempre in termini di stringa per
     * evitare accessi lunghi a oggetti synchronized
     *
     * @param dest Destinations
     * @return ArrayList
     */
    protected synchronized ArrayList listEvaluations(Destinations dest )
    {
        ArrayList arl = new ArrayList();
        if(dest==null)
            return arl;
        ArrayList eval = dest.GetEvaluations();
        if(eval.size()<=0)
            return arl;
        Evaluations curEval;
        for(int i=0;i<eval.size();i++){
            curEval = (Evaluations)eval.get(i);
            arl.add(String.valueOf(i)+"-"+curEval.getVoto());
            arl.add("   "+curEval.getGiudizio());
        }
        return arl;
    }

    /**
     * getBestDestination
     * Visualizza i dati della miglior destinazione presente sul server
     *
     * @return String La stringa di descrizione
     */
    protected synchronized String getBestDestination()
    {
        if(TheServer.GetBestDestinations()==null)
            return "Non e'stata votata ancora nessuna destinazione";
        return "La destinazione meglio votata e'"+TheServer.GetBestDestinations().getName()+
                " con valutazione "+String.valueOf(TheServer.GetBestDestinations().getAverage());
    }


    /**
     * run
     * Override standard per thread
     */
    public void run()
    {
        curState = 0;
        waitForVars = 0;
        String line;
        String name;
        try {
            clientSocket.setSoTimeout(TheServer.MAX_TIMEOUT_MILLISEC);
            is = new DataInputStream(clientSocket.getInputStream());
            os = new PrintStream(clientSocket.getOutputStream());

            while (true) {
                setTimeStamp();
                line = "";
                if ((curState != 3 && curState != 7 &&
                     curState != 6 && curState!=8 && curState != 2 &&
                     curState != 1) ||
                    waitForVars > 0){
                    line = is.readLine();
                    if(line.length()>80)
                        line = line.substring(0,80);
                }

                if (line != null){
                    if (line.equalsIgnoreCase("fine"))
                        break;
                }
                if (waitForVars > 0) {
                    System.out.println("Adding var "+line + " for state "+String.valueOf(curState)+ " on "+String.valueOf(waitForVars));
                    varsToWaitFor[waitForVars] = line;
                    waitForVars--;
                }else{

                    messages = "";
                    switch (curState) {
                    case (1000): {
                        if (waitForVars == 0)
                            curState = 0;
                    }
                    case (0): { //Login
                        if (line.equalsIgnoreCase("login") && waitForVars == 0) {
                            waitForVars = 2;
                            curState = 1;
                            break;
                        } else if (line.equalsIgnoreCase("nuova") &&
                                   waitForVars == 0) {
                            waitForVars = 1;
                            curState = 2;
                            break;
                        } else if (line.equalsIgnoreCase("valuta") &&
                                   waitForVars == 0) {
                            waitForVars = 0;
                            curState = 3;
                            break;
                        } else if (line.equalsIgnoreCase("scelta") &&
                                   waitForVars == 0) {
                            waitForVars = 0;
                            curState = 8;
                            break;
                        }

                        break;
                    }
                    case (1): {
                        System.out.println("Inserito utente " + varsToWaitFor[1]);
                        curUser = insertUser(varsToWaitFor[1], varsToWaitFor[2]);
                        if (curUser == null) {
                            waitForVars = 2;
                            os.println(false);
                        } else {
                            os.println(true);
                            curState = 0;
                        }
                        break;
                    }
                    case (2): {
                        System.out.println("Inserita meta " + varsToWaitFor[1]);
                        curDestination = insertDestination(varsToWaitFor[1]);
                        os.println(messages);
                        curState = 0;
                        break;
                    }
                    case (3): {
                        ArrayList arl = listDestinations();
                        os.println(arl.size());
                        os.flush();
                        for (int k = 0; k < arl.size(); k++) {
                            os.println((String) arl.get(k));
                            os.flush();
                        }
                        curState = 7;
                        waitForVars = 1;
                        break;
                    }
                    case (5): {

                        if (line.equalsIgnoreCase("si")) {
                            curState = 6;
                            waitForVars = 2;
                            break;
                        }
                        curState = 0;
                        waitForVars = 0;
                        break;
                    }
                    case (6): {
                        insertEvaluation(curDestination, varsToWaitFor[2],
                                         varsToWaitFor[1]);
                        System.out.println("Inserita valutazione " +
                                           varsToWaitFor[1] + "," +
                                           varsToWaitFor[2] + " per " +
                                           curDestination.getName());
                        curState = 0;
                        waitForVars = 0;

                        break;
                    }

                    case (7): {
                        int metasel = 0;

                        try {
                            metasel=Integer.parseInt(varsToWaitFor[1]);
                            curDestination = getDestination(metasel);
                        } catch (NumberFormatException e) {
                            curDestination = getDestination(varsToWaitFor[1]);
                        }

                        ArrayList arl = listEvaluations(curDestination);
                        System.out.println("Giudizi "+arl.size());
                        os.println(arl.size());

                        os.flush();
                        for (int k = 0; k < arl.size(); k++) {
                            os.println("---");
                            os.println((String) arl.get(k));
                            os.flush();
                        }
                        curState = 5;
                        waitForVars = 0;
                        break;
                    }
                    case (8): {
                        os.println(getBestDestination());
                        os.flush();
                        curState = 0;
                        waitForVars = 0;
                        break;
                    }

                    default:
                        break;
                    }
                }
            }

           /* os.flush();
            is.close();
            os.close();
            clientSocket.close();*/

       }catch(java.net.SocketTimeoutException en){
           if(curUser!=null)
               System.out.println("Raggiunto timeout, utente "+curUser.getLogin()+" disconnesso");
           else
               System.out.println("Raggiunto timeout, utente disconnessione");
       }catch (IOException e) {

        }

        closeSockets();
    }

    public void closeSockets()
    {
        try{
            os.flush();
            is.close();
            os.close();
            clientSocket.close();
            connectedListenersArray[selfListIndex]=null;
        }catch(Exception e){

        }
    }

    public void setTimeStamp() {
        this.timeStamp = new Date();
    }

    public  Date getTimeStamp() {
        System.out.println("ERTIMESTAMP");
        return timeStamp;
    }

    private Date timeStamp;
}

